/*
 * gAVLTree.h: Header file for gAVLTrees.
 * Copyright (C) 1998,2001  Michael H. Buselli
 * This is version 0.1.3 (alpha).
 * Generated from $Id: xAVLTree.h.sh,v 1.5 2001/06/07 06:58:28 cosine Exp $
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * The author of this library can be reached at the following address:
 * Michael H. Buselli
 * 30051 N. Waukegan Rd. Apt. 103
 * Lake Bluff, IL  60044-5412
 *
 * Or you can send email to <cosine@cosine.org>.
 * The official web page for this product is:
 * http://www.cosine.org/project/AVLTree/
 */

#ifndef _GAVLTREE_H_
#define _GAVLTREE_H_

/* typedef the keytype */
typedef const void *gAVLKey;

/*
 * Comparison function type for gAVLTrees.  It is important that for any
 * A, B, C, and D objects that if AVLCompare(A, B) < 0, AVLCompare(B, C)
 * < 0, and AVLCompare(A, D) == 0, then the following are also true:
 * AVLCompare(A, C) < 0, AVLCompare(B, A) > 0, AVLCompare(D, A) == 0.
 * This code cannot verify that the function supplied by the user has
 * this property--this is the coder's responsibility--but the code will
 * by default perform some sanity checks at the cost of performance.
 * Turn the sanity checks off in avlconfig.h by defining AVL_NO_SANITY.
 */
typedef int (*gAVLCompare)(gAVLKey a, gAVLKey b);

/* Comparison function for gAVLTrees is defined by the user for each
 * individual gAVLTree.
 */
#define gAVLKey_cmp(tree, a, b) (((tree)->cmp)((a), (b)))


typedef struct _gAVLNode {
  gAVLKey key;
  long depth;
  void *item;
  struct _gAVLNode *parent;
  struct _gAVLNode *left;
  struct _gAVLNode *right;
} gAVLNode;


typedef struct {
  gAVLNode *top;
  long count;
  gAVLKey (*getkey)(const void *item);
  gAVLCompare cmp;
} gAVLTree;


typedef struct {
  const gAVLTree *avltree;
  const gAVLNode *curnode;
} gAVLCursor;


extern gAVLTree *gAVLAllocTree (gAVLKey (*getkey)(void const *item),
        gAVLCompare cmp);
extern void gAVLFreeTree (gAVLTree *avltree, void (freeitem)(void *item));
extern int gAVLInsert (gAVLTree *avltree, void *item);
extern void *gAVLSearch (gAVLTree const *avltree, gAVLKey key);
extern int gAVLDelete (gAVLTree *avltree, gAVLKey key);
extern void *gAVLFirst (gAVLCursor *avlcursor, gAVLTree const *avltree);
extern void *gAVLNext (gAVLCursor *avlcursor);

#endif
